//
//  ViewController.swift
//  DigitalOutput
//
//  Created by Phidgets on 2018-02-06.
//  Copyright © 2018 Phidgets. All rights reserved.
//

import UIKit
import Phidget22Swift

class ViewController: UIViewController {
    
    let ch = DigitalOutput()
    var phidgetInfoBox:PhidgetInfoBoxViewController! = nil
    var pickerArray = [String]()
    
    @IBOutlet var dutyCycleSlider: UISlider!
    @IBOutlet var dutyCycleLabel: UILabel!
    @IBOutlet var dutyCycleButton: UIButton!
    @IBOutlet var currentLimitTxt: UILabel!
    @IBOutlet var currentLimitSlider: UISlider!
    @IBOutlet var currentLimitLabel: UILabel!
    @IBOutlet var forwardVoltageButton: UIButton!
    @IBOutlet var stackView: UIStackView!
    
    
    override func viewDidLoad() {
        super.viewDidLoad()
        do {
            try Net.enableServerDiscovery(serverType: .deviceRemote)
            let _ = ch.attach.addHandler(attach_handler)
            let _ = ch.detach.addHandler(detach_handler)
            let _ = ch.error.addHandler(error_handler)
            try ch.open()
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    //MARK: Event Handlers
    func attach_handler(sender: Phidget){
        do{
            try phidgetInfoBox.fillPhidgetInfo(phid: sender)
            let deviceID = try sender.getDeviceID()
            let channelSubClass = try sender.getChannelSubclass()
            let attachedDevice = sender as! DigitalOutput
            
            if(deviceID == DeviceID.PN_1031 || deviceID == DeviceID.PN_1032 || deviceID == DeviceID.PN_LED1000){
                let maxLEDCurrentLimit = try Float(attachedDevice.getMaxLEDCurrentLimit())
                let minLEDCurrentLimit = try Float(attachedDevice.getMinLEDCurrentLimit())
                let ledCurrentLimit = try Float(attachedDevice.getLEDCurrentLimit())
                
                DispatchQueue.main.async {
                    self.stackView.isHidden = false
                    self.currentLimitLabel.isHidden = false
                    self.currentLimitSlider.isHidden = false
                    self.currentLimitTxt.isHidden = false
                    self.forwardVoltageButton.isHidden = false
                    self.currentLimitSlider.minimumValue = minLEDCurrentLimit*1000 //convert to mA
                    self.currentLimitSlider.maximumValue = maxLEDCurrentLimit*1000
                    self.currentLimitSlider.value = ledCurrentLimit*1000
                    self.currentLimitLabel.text = String(format: "%.0f mA", ledCurrentLimit*1000)
                    if(deviceID == DeviceID.PN_LED1000){
                        self.pickerArray = ["3.2 Volts","4.0 Volts", "4.8 Volts", "5.6 Volts"]
                    }
                    else{
                        self.pickerArray = ["1.7 Volts", "2.75 Volts", "3.9 Volts", "5.0 Volts"]
                    }
                }
            }
            else{
                DispatchQueue.main.async {
                    self.stackView.isHidden = false
                    self.currentLimitLabel.isHidden = true
                    self.currentLimitSlider.isHidden = true
                    self.currentLimitTxt.isHidden = true
                    self.forwardVoltageButton.isHidden = true
                }
            }
            
            if(channelSubClass == ChannelSubclass.digitalOutputDutyCycle || channelSubClass == ChannelSubclass.digitalOutputLEDDriver){
                let dutyCycle = try Float(attachedDevice.getDutyCycle())
                DispatchQueue.main.async {
                    self.dutyCycleSlider.isEnabled = true
                    self.dutyCycleSlider.value = dutyCycle
                    self.dutyCycleLabel.text = "\(dutyCycle)"
                }
            }
            else{
                dutyCycleSlider.isEnabled = false
                let currentState = try attachedDevice.getState()
                DispatchQueue.main.async {
                    self.dutyCycleSlider.setValue((currentState == true) ? 1.0:0.0, animated: false)
                    self.dutyCycleLabel.text = (currentState == true) ? "1.00" : "0.00"
                    self.dutyCycleButton.setTitle((currentState == true) ? "Off" : "On", for: UIControlState.normal)
                }
            }
            
            
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    func detach_handler(sender: Phidget){
        do{
            try phidgetInfoBox.fillPhidgetInfo(phid: nil)
            DispatchQueue.main.async {
                self.stackView.isHidden = true
            }
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    func error_handler(sender: Phidget, data: (code: ErrorEventCode, description: String)){
        self.outputErrorMessage(errorTitle: "Error Code: \(data.code)", errorMessage: data.description)
    }
    
    var timeStamp:TimeInterval = 0
    var lastTimeStamp:TimeInterval = 0
    func outputErrorMessage(errorTitle:String, errorMessage:String){
        timeStamp = NSDate.timeIntervalSinceReferenceDate
        //Add a delay for error events so the user does not get bombarded by error events
        if(timeStamp - lastTimeStamp > 5){
            let alert = UIAlertController(title: errorTitle, message: errorMessage, preferredStyle: .alert)
            alert.addAction(UIAlertAction(title: "OK", style: .default, handler: {action -> Void in
            }))
            self.present(alert, animated: true, completion: nil)
            lastTimeStamp = timeStamp
        }
    }
    
    //MARK: GUI Controls
    @IBAction func dutyCycleChanged(_ sender: Any) {
        let slider = sender as! UISlider
        do{
            try ch.setDutyCycle(Double(slider.value))
            dutyCycleLabel.text = String(format:"%0.2f",slider.value)
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    @IBAction func dutyCycleOnOff(_ sender: Any) {
        let button = sender as! UIButton
        do{
            let state = try (ch.getState() == true) ? false : true
            try ch.setState(state)
            dutyCycleSlider.setValue((state == true) ? 1.0 : 0.0, animated: false)
            dutyCycleLabel.text = (state == true) ? "1.00" : "0.00"
            button.setTitle((state == true) ? "Off" : "On", for: UIControlState.normal)
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    @IBAction func setCurrentLimit(_ sender: Any) {
        let slider = sender as! UISlider
        do{
            try ch.setLEDCurrentLimit(Double(slider.value/1000.0))
            currentLimitLabel.text = String(format:"%d mA",Int(slider.value))
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    //MARK: Navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if segue.identifier == "infoBoxSegue" {
            phidgetInfoBox = segue.destination as! PhidgetInfoBoxViewController
        }
        else if segue.identifier == "forwardVoltageSegue"{
            let vc:PickerViewController = segue.destination as! PickerViewController
            dimBackground()
            vc.titleName = "Forward Voltage"
            vc.pickerArray = pickerArray
            vc.ch = ch
        }
    }
    
    @IBAction func forwardVoltageExit(_ segue: UIStoryboardSegue){
        dimBackground()
    }
    
    func dimBackground(){
        self.view.alpha = (self.view.alpha == 1.0) ? 0.5 : 1.0
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
    }
}

